import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { json, scaleLinear, scaleTime, extent, select, scaleOrdinal } from 'd3';

const jsonUrl = 'https://raw.githubusercontent.com/freeCodeCamp/ProjectReferenceData/master/cyclist-data.json';
const width = 900;
const height = 450;
const margin = { top: 100, bottom: 90, left: 80, right: 100 };

const getTimeStamp = (seconds) => {
 	var minutes = Math.floor(seconds / 60);
  var sec = seconds % 60;
  var str = '';
  if (minutes < 10) {
    str += '0' + minutes;
  } else {
    str += minutes;
  }
  str += ':';
  if (sec < 10) {
    str += '0' + sec;
  } else {
    str += sec;
  }
  return str;
}

const App = () => {
  const [data, setData] = useState(null);
  
  useEffect(() => {
  	json(jsonUrl).then(setData);
  }, []);
  
  if (!data) {
    return <div>Loading...</div>;
  }
  
  for(let i=0; i<data.length; i++) {
   	if (data[i].Doping === '') {
      data[i].didDope = 'No doping allegations';
    } else {
      data[i].didDope = 'Riders with doping allegations';
    }
  }
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const xScale = scaleLinear().domain(extent(data, d => d.Year)).range([0, innerWidth]).nice();
  const yScale = scaleLinear().domain(extent(data, d => d.Seconds)).range([0, innerHeight]);
  const colorScale = scaleOrdinal().domain(extent(data, d => d.didDope)).range(['blue','red']);
  
  return (
    <>
  	<div id='title'>
      <h1>Doping in Professional Bicycle Racing</h1>
      <h3>35 Fastest times up Alpe d'Huez</h3>
    </div>
      <svg width={width} height={height}>
        <g transform={`translate(${margin.left},${margin.top})`}>
        {
        	xScale.ticks().map(tickValue => <g transform={`translate(${xScale(tickValue)},${innerHeight+5})`}>
                               <text
                                 style={{textAnchor: 'middle'}}
                                 dy="0.71em"
                                 >{tickValue}</text>
                             </g>)
        }
        {
        	yScale.ticks().map(tickValue => <g transform={`translate(-5,${yScale(tickValue)})`}>
                               <text
                                 style={{textAnchor: 'end'}}
                                 dy="0.32em"
                                 >{getTimeStamp(tickValue)}</text>
                             </g>)  
        }
        <g id='y-axis'>
        <line 
          y2={innerHeight}
          stroke='black'
          />
          </g>
        <g id='x-axis' transform={`translate(0,${innerHeight})`}>
          <line 
            x2={innerWidth}
            stroke='black'
            />
        </g>
          <text
            transform={`translate(-50,${innerHeight/2}) rotate(-90)`}
            id='axis-label'
            >Time in Minutes
          </text>
          {
          	data.map(d => <g onMouseEnter={() => {
                       	select('#tooltip').append('span').text(d.Name + ' : ' + d.Nationality);
                        select('#tooltip').append('span').text('Year: ' + d.Year + ', Time: ' + d.Time);
   											select('#tooltip').append('span').text(d.Doping);                   
                        select('#tooltip').style('left',margin.left + xScale(d.Year) + 10 + 'px');
                        select('#tooltip').style('top',margin.top + yScale(d.Seconds) + 70 + 'px');
                       	select('#tooltip').style('visibility','visible');
                       }}
                            onMouseLeave={() => {
                       	select('#tooltip').selectAll('span').remove();
                        select('#tooltip').style('visibility','hidden');
                       }}>
                     		<circle 
                          className='dot'
                          cx={xScale(d.Year)}
                          cy={yScale(d.Seconds)}
                          r={6}
                          fill={colorScale(d.didDope)}
                          />
                     </g>) 
          }
        </g>
        <g id='legend' transform={`translate(${width - 50},${margin.top - 50})`}>
          {
            colorScale.domain().map((tickValue, i) => <g transform={`translate(40,${(i+1)*30})`}>
                                    
                                      <text
                                        x={-15}
                                        style={{textAnchor: 'end'}}
                                        dy="0.71em"
                                        >{tickValue}</text>
                                      
                                      <rect 
                                     width={15}
                                     height={15}
                                     fill={colorScale(tickValue)}
                                     />
                                   </g>)
          }
        </g>
      </svg>
      <div id='tooltip'></div>
    </>
  );
  
}

ReactDOM.render(<App />, document.getElementById('root'));
